from AccessControl import ClassSecurityInfo, IUserFolder
from OFS.SimpleItem import SimpleItem
from Globals import InitializeClass
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

import urllib

class DuplicateError(Exception):
    pass

_manage_permission = 'Manage Local UserManager'

class LocalUserManager(SimpleItem):
    """Local usermanagement system for Zope
    
    Can be used to manage a plain userfolder from another 
    location in the Zope object tree.
    """

    security = ClassSecurityInfo()
    meta_type = 'Local User Manager'

    manage_options = (({'label': 'Edit', 'action': 'edit_tab'},) + 
                        SimpleItem.manage_options)

    manage_main = edit_tab = PageTemplateFile('www/mainUser', 
                                                globals(), 
                                                __name__ = 'edit_tab')

    add_user = PageTemplateFile('www/addUser',
                                    globals(),
                                    __name__ = 'add_user_form')

    edit_user = PageTemplateFile('www/editUser',
                                    globals(),
                                    __name__ = 'edit_user')

    def __init__(self, id, title, master):
        self._usernames = []
        self.id = id
        self.title = title
        self._master = master

    def manage_afterAdd(self, item, container):
        if not container.restrictedTraverse(self._master):
            raise LookupError, 'Master userfolder not found!'
        LocalUserManager.inheritedAttribute('manage_afterAdd')(self, item, container)

    security.declareProtected(_manage_permission, 'userFolderEditUser')
    def userFolderEditUser(self, name, password, roles, domains, **kw):
        """Edits a user"""
        if name not in self._usernames:
            raise NameError, '%s is not a known user' % name
        self._get_master().userFolderEditUser(name, password, [], domains, **kw)

        # now add the local roles and delete some if necessary
        self.aq_inner.aq_parent.manage_delLocalRoles([name])
        if roles:
            self.aq_inner.aq_parent.manage_addLocalRoles(name, roles)

    security.declareProtected(_manage_permission, 'userFolderDelUsers')
    def userFolderDelUsers(self, names):
        """Deletes a user"""
        for name in names:
            if name not in self._usernames:
                raise NameError, '%s is not a known user' % name
        for name in names:
            self._usernames.remove(name)
        self.aq_inner.aq_parent.manage_delLocalRoles(names)
        self._get_master().userFolderDelUsers(names)
        self._p_changed = 1

    security.declareProtected(_manage_permission, 'userFolderAddUser')
    def userFolderAddUser(self, name, password, roles, domains, **kw):
        """Adds a user"""
        self._sync()
        master = self._get_master()
        if name in self._usernames:
            raise DuplicateError, 'Name %s already in use' % name
        if name in master.getUserNames():
            message = 'Name %s already defined on the master userfolder' % name
            if 'REQUEST' in kw.keys():
                raise DuplicateError, message
            else:
                return self.manage_main(
                    manage_tabs_message=message)
        master.userFolderAddUser(name, password, [], domains, **kw)
        if roles:
            self.aq_inner.aq_parent.manage_addLocalRoles(name, roles)
        self._usernames.append(name)
        self._p_changed = 1
        
        return self.manage_main(manage_tabs_message='User added')

    security.declareProtected(_manage_permission, 'getUsers')
    def getUsers(self):
        """Returns a sequence of all userobjects"""
        self._sync()
        users = []
        for name in self._usernames:
            users.append(self._get_master().getUser(name))
        return users

    security.declareProtected(_manage_permission, 'getUserNames')
    def getUserNames(self):
        """Returns a sequence of all usernames"""
        self._sync()
        return self._usernames[:]

    security.declareProtected(_manage_permission, 'getUser')
    def getUser(self, name):
        """Returns a userobject"""
        if name not in self._usernames:
            raise NameError, '%s is not a known user' % name
        return self._get_master().getUser(name)

    security.declareProtected(_manage_permission, 'getRoles')
    def getRoles(self, name):
        """Returns a list of all local roles for this user"""
        if name not in self._usernames:
            raise NameError, '%s is not a known user' % name
        roles = self.getUser(name).getRolesInContext(self.aq_inner.aq_parent)
        return roles

    security.declareProtected(_manage_permission, 'getDomains')
    def getDomains(self, name):
        """Returns a list of domains"""
        return self._get_master().getDomains(name)

    def _get_master(self):
        """Returns a reference to the master userfolder"""
        return self.restrictedTraverse(self._master)

    def _sync(self):
        """Syncs with the master userfolder"""
        master_names = self._get_master().getUserNames()
        for name in self._usernames:
            if name not in master_names:
                self._usernames.remove(name)

    security.declareProtected(_manage_permission, 'manage_addUser')
    def manage_addUser(self, REQUEST):
        """Add a user"""
        if not REQUEST.has_key('password') or not REQUEST.has_key('confirm'):
            return self.add_user(
                manage_tabs_message='Enter password and confirm fields')
        if REQUEST['password'] != REQUEST['confirm']:
            return self.add_user(
                manage_tabs_message='Password and confirmation don\'t match')
        password = REQUEST['password']
        username = REQUEST['name']
        if not username:
            return self.add_user(
                manage_tabs_message='No name supplied!')
        
        domains = []
        if REQUEST.has_key('domains'):
            domains = REQUEST['domains']

        roles = []
        if REQUEST.has_key('roles'):
            roles = REQUEST['roles']

        try:
            self.userFolderAddUser(username, password, roles, domains, REQUEST=REQUEST)
        except DuplicateError, e:
            return self.manage_main(
                manage_tabs_message=e)

        return self.manage_main(manage_tabs_message='User %s added' % username)

    security.declareProtected(_manage_permission, 'manage_delUser')
    def manage_delUser(self, REQUEST):
        """Delete a user"""
        if not REQUEST.has_key('names') or not REQUEST['names']:
            return self.manage_main(
                manage_tabs_message='No username(s) provided')
        usernames = REQUEST['names']
        self.userFolderDelUsers(usernames)
        return self.manage_main(
            manage_tabs_message='User(s) %s deleted' % ', '.join(usernames))

    security.declareProtected(_manage_permission, 'manage_editUser')
    def manage_editUser(self, REQUEST):
        """Edit a user"""
        password = None
        if ((REQUEST.has_key('password') and not REQUEST.has_key('confirm')) or
            (REQUEST.has_key('confirm') and not REQUEST.has_key('password')) or
            (REQUEST.has_key('password') and REQUEST.has_key('confirm') and
              REQUEST['password'] != REQUEST['confirm'])):
            return self.edit_user(
              manage_tabs_message='Password and confirmation don\'t match!')
        if REQUEST.has_key('password') and REQUEST['password'].strip():
            password = REQUEST['password']
        roles = []
        if REQUEST.has_key('roles') and REQUEST['roles']:
            roles = REQUEST['roles']
        domains = []
        if REQUEST.has_key('domains') and REQUEST['domains']:
            domains = REQUEST['domains']
        try:
            self.userFolderEditUser(REQUEST['name'], password, roles, domains)
        except NameError, e:
            return self.manage_main(
                manage_tabs_message=e)
        return self.manage_main(
            manage_tabs_message='Userdata changed')
        
InitializeClass(LocalUserManager)

manage_addLocalUserManagerForm = PageTemplateFile(
    "www/addLocalUserManager",
    globals(),
    __name__ = 'manage_addLocalUserManagerForm')

def manage_addLocalUserManager(self, id, title, master, REQUEST=None):
    """Add a local userfolder to the current container"""
    id = self._setObject(id, LocalUserManager(id, title, master))
    if REQUEST is None:
        return ''
    try:
        u = self.DestinationURL()
    except:
        u = REQUEST['URL1']
    if REQUEST.has_key('submit_edit'):
        u = '%s/%s' % (u, urllib.quote(id))
    REQUEST.RESPONSE.redirect(u + '/manage_main')
    return ''

def find_userfolders(self):
    root = self.restrictedTraverse('/')
    userfolders = []

    obj = self.restrictedTraverse(self.getPhysicalPath())
    while 1:
        if 'acl_users' in obj.objectIds():
            userfolders.append('/'.join(obj.acl_users.getPhysicalPath()))
        if obj == root:
            break
        obj = obj.aq_inner.aq_parent
    
    userfolders.sort()

    return userfolders

def _find_userfolders_helper(obj, userfolders):
    for o in obj.objectValues():
        if o.id == 'acl_users':
            userfolders.append('/'.join(o.getPhysicalPath()))
        elif hasattr(o, 'objectValues'):
            _find_userfolders_helper(o, userfolders)
